%% clear
clc;
clear;
close all;
format compact;
format short g;

%% inital stuff
fem = getInitFem;
prob.fem.dof = getDof(fem);

prob.theta(1).fun = @(mu) mu(1);
prob.theta(2).fun = @(mu) mu(2);

prob = performPreAssembling(fem,prob);

%% example for solving
mu = [.1;1]

tic; uCom = getSol(prob,mu,fem); toc
tic; uPre = getSol(prob,mu); toc

figure(1)
fem.sol = femsol(uCom);
subplot(1,3,1); postplot(fem,'tridata','u');

fem.sol = femsol(uPre);
subplot(1,3,2); postplot(fem,'tridata','u');

fem.sol = femsol(abs(uPre-uCom));
subplot(1,3,3); postplot(fem,'tridata','u');

%% compute matrices for discrete inner product
[fem.const{2:2:end}] = deal(1);
[C,D] = assemble(fem,'Out',{'C','D'});

L2 = D;
L2 = (L2+L2')/2;

H1 = C+D;
H1 = (H1+H1')/2;

prob.fem.innProd.L2 = L2;
prob.fem.innProd.H1 = H1;

% % example for intended usage
% norm_X = 'L2';
% figure(2);
% spy(prob.fem.innProd.(norm_X));

%% Aufgabe 1: POD

% parameter space w.r.t. \mu_1
D = [.01 100];

% number of snapshots n_train=2^j_train+1
j_train = 7;
n_train = 2^j_train + 1;

% parameter training set
Xi_train = [logspace(log10(D(1)),log10(D(2)),2^j_train+1); ...
            ones(1,2^j_train+1)];

% compute snapshots
U = zeros(prob.fem.dof.num,2^j_train+1);
for n=1:2^j_train+1
  U(:,n) = getSol(prob,Xi_train(:,n));
end

% compute correlation w.r.t. H1
norm_X = 'H1';
C = U'*prob.fem.innProd.(norm_X)*U;
C = 1./n_train*C;

[Psi,Lambda] = computePOD(C,U);

% test for orthonormality
Psi'*prob.fem.innProd.(norm_X)*Psi

%% plot POD basis functions
% figure(3);
% for l=1:size(Psi,2)
%   fem.sol = femsol(Psi(:,l));
%   postplot(fem,'tridata','u');
%   title(num2str(l,'l = %d'));
%   pause;
% end

%% Aufgabe 2: compute projection error for different truncations
d = size(Psi,2);
err = zeros(d,size(U,2));
for l=1:d
  U_POD = Psi(:,1:l)*(Psi(:,1:l)'*prob.fem.innProd.(norm_X)*U);
  U_diff = U-U_POD;
  for n=1:size(U,2)
    err(l,n) = U_diff(:,n)'*prob.fem.innProd.(norm_X)*U_diff(:,n);
  end
end

% verify Lemma 1.5.3
[1./n_train * sum(err,2) trace(C)-cumsum(Lambda)]

figure(4);
loglog(Xi_train(1,:)',err','x');
legend(num2str((1:d)','l=%d'));
xlabel ('$\mu$','Interpreter','LaTex');
ylabel('$|| u ( \mu ) - u_{N,proj}^{ POD}\  (\mu) ||_X$','Interpreter','LaTex');


%% Aufgabe 3: compute POD error in dependency of n_train
err = zeros(0,j_train);
for j=1:j_train 
  ind = 1:2^(j_train-j):2^j_train+1;
  C_ind = C(ind,ind);
  U_ind = U(:,ind);

  [Psi,Lambda] = computePOD(C_ind,U_ind);
  d = size(Psi,2);

  if d>size(err,1)
    err = [err; NaN*ones(d-size(err,1),j_train)];
  end
  
  err(1:d,j) = 1/(2^j+1) * (trace(C_ind)-cumsum(Lambda));
end
err(err<=0) = eps;


figure(5);
loglog(2.^(1:j_train)+1,err','x-');
legend(num2str((1:size(err,1))','N=%d'));
xlabel n_{train};
ylabel \epsilon_N^{POD};


